package sun.security.util;

import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.TimeZone;
import java.util.Date;
import java.util.Arrays;
import java.util.Comparator;
import java.math.BigInteger;
import java.io.IOException;
import java.io.ByteArrayOutputStream;

public class DerOutputStream extends ByteArrayOutputStream implements DerEncoder
{
    private static ByteArrayLexOrder lexOrder;
    private static ByteArrayTagOrder tagOrder;
    
    public DerOutputStream(int size) {
        super(size);
    }
    
    public DerOutputStream() {
    }
    
    public void write(byte b, byte[] b2) throws IOException {
        super.write((int)b);
        this.putLength(b2.length);
        super.write(b2, 0, b2.length);
    }
    
    public void write(byte b, DerOutputStream derOutputStream) throws IOException {
        super.write((int)b);
        this.putLength(derOutputStream.count);
        super.write(derOutputStream.buf, 0, derOutputStream.count);
    }
    
    public void writeImplicit(byte b, DerOutputStream derOutputStream) throws IOException {
        super.write((int)b);
        super.write(derOutputStream.buf, 1, derOutputStream.count - 1);
    }
    
    public void putDerValue(DerValue derValue) throws IOException {
        derValue.encode(this);
    }
    
    public void putBoolean(boolean b) throws IOException {
        super.write(1);
        this.putLength(1);
        if (b) {
            super.write(255);
        }
        else {
            super.write(0);
        }
    }
    
    public void putEnumerated(int n) throws IOException {
        super.write(10);
        this.putIntegerContents(n);
    }
    
    public void putInteger(BigInteger bigInteger) throws IOException {
        super.write(2);
        byte[] byteArray = bigInteger.toByteArray();
        this.putLength(byteArray.length);
        super.write(byteArray, 0, byteArray.length);
    }
    
    public void putInteger(Integer n) throws IOException {
        this.putInteger((int)n);
    }
    
    public void putInteger(int n) throws IOException {
        super.write(2);
        this.putIntegerContents(n);
    }
    
    private void putIntegerContents(int n) throws IOException {
        byte[] array = new byte[4];
        int i = 0;
        array[3] = (byte)(n & 0xFF);
        array[2] = (byte)((n & 0xFF00) >>> 8);
        array[1] = (byte)((n & 0xFF0000) >>> 16);
        array[0] = (byte)((n & 0xFF000000) >>> 24);
        if (array[0] == -1) {
            for (int n3 = 0; n3 < 3 && array[n3] == -1 && (array[n3 + 1] & 0x80) == 0x80; ++n3) {
                ++i;
            }
        }
        else if (array[0] == 0) {
            for (int n2 = 0; n2 < 3 && array[n2] == 0 && (array[n2 + 1] & 0x80) == 0x0; ++n2) {
                ++i;
            }
        }
        this.putLength(4 - i);
        for (; i < 4; ++i) {
            super.write((int)array[i]);
        }
    }
    
    public void putBitString(byte[] b) throws IOException {
        super.write(3);
        this.putLength(b.length + 1);
        super.write(0);
        this.write(b);
    }
    
    public void putUnalignedBitString(BitArray bitArray) throws IOException {
        byte[] byteArray = bitArray.toByteArray();
        super.write(3);
        this.putLength(byteArray.length + 1);
        super.write(byteArray.length * 8 - bitArray.length());
        this.write(byteArray);
    }
    
    public void putTruncatedUnalignedBitString(BitArray bitArray) throws IOException {
        this.putUnalignedBitString(bitArray.truncate());
    }
    
    public void putOctetString(byte[] array) throws IOException {
        this.write((byte)4, array);
    }
    
    public void putNull() throws IOException {
        super.write(5);
        this.putLength(0);
    }
    
    public void putOID(ObjectIdentifier objectIdentifier) throws IOException {
        objectIdentifier.encode(this);
    }
    
    public void putSequence(DerValue[] array) throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        for (int i = 0; i < array.length; ++i) {
            array[i].encode(derOutputStream);
        }
        this.write((byte)48, derOutputStream);
    }
    
    public void putSet(DerValue[] array) throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        for (int i = 0; i < array.length; ++i) {
            array[i].encode(derOutputStream);
        }
        this.write((byte)49, derOutputStream);
    }
    
    public void putOrderedSetOf(byte b, DerEncoder[] array) throws IOException {
        this.putOrderedSet(b, array, DerOutputStream.lexOrder);
    }
    
    public void putOrderedSet(byte b, DerEncoder[] array) throws IOException {
        this.putOrderedSet(b, array, DerOutputStream.tagOrder);
    }
    
    private void putOrderedSet(byte b, DerEncoder[] array, Comparator<byte[]> c) throws IOException {
        DerOutputStream[] array2 = new DerOutputStream[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = new DerOutputStream();
            array[i].derEncode(array2[i]);
        }
        byte[][] a = new byte[array2.length][];
        for (int j = 0; j < array2.length; ++j) {
            a[j] = array2[j].toByteArray();
        }
        Arrays.sort(a, c);
        DerOutputStream derOutputStream = new DerOutputStream();
        for (int k = 0; k < array2.length; ++k) {
            derOutputStream.write(a[k]);
        }
        this.write(b, derOutputStream);
    }
    
    public void putUTF8String(String s) throws IOException {
        this.writeString(s, (byte)12, "UTF8");
    }
    
    public void putPrintableString(String s) throws IOException {
        this.writeString(s, (byte)19, "ASCII");
    }
    
    public void putT61String(String s) throws IOException {
        this.writeString(s, (byte)20, "ISO-8859-1");
    }
    
    public void putIA5String(String s) throws IOException {
        this.writeString(s, (byte)22, "ASCII");
    }
    
    public void putBMPString(String s) throws IOException {
        this.writeString(s, (byte)30, "UnicodeBigUnmarked");
    }
    
    public void putGeneralString(String s) throws IOException {
        this.writeString(s, (byte)27, "ASCII");
    }
    
    private void writeString(String s, byte b, String charsetName) throws IOException {
        byte[] bytes = s.getBytes(charsetName);
        super.write((int)b);
        this.putLength(bytes.length);
        this.write(bytes);
    }
    
    public void putUTCTime(Date date) throws IOException {
        this.putTime(date, (byte)23);
    }
    
    public void putGeneralizedTime(Date date) throws IOException {
        this.putTime(date, (byte)24);
    }
    
    private void putTime(Date date, byte b) throws IOException {
        TimeZone timeZone = TimeZone.getTimeZone("GMT");
        String pattern;
        if (b == 23) {
            pattern = "yyMMddHHmmss'Z'";
        }
        else {
            b = 24;
            pattern = "yyyyMMddHHmmss'Z'";
        }
        SimpleDateFormat o = new SimpleDateFormat(pattern, Locale.US);
        o.setTimeZone(timeZone);
        byte[] bytes = o.format(date).getBytes("ISO-8859-1");
        super.write((int)b);
        this.putLength(bytes.length);
        this.write(bytes);
    }
    
    public void putLength(int n) throws IOException {
        if (n < 128) {
            super.write((int)(byte)n);
        }
        else if (n < 256) {
            super.write(-127);
            super.write((int)(byte)n);
        }
        else if (n < 65536) {
            super.write(-126);
            super.write((int)(byte)(n >> 8));
            super.write((int)(byte)n);
        }
        else if (n < 16777216) {
            super.write(-125);
            super.write((int)(byte)(n >> 16));
            super.write((int)(byte)(n >> 8));
            super.write((int)(byte)n);
        }
        else {
            super.write(-124);
            super.write((int)(byte)(n >> 24));
            super.write((int)(byte)(n >> 16));
            super.write((int)(byte)(n >> 8));
            super.write((int)(byte)n);
        }
    }
    
    public void putTag(byte b, boolean b2, byte b3) {
        int b4 = (byte)(b | b3);
        if (b2) {
            b4 = (byte)(b4 | 0x20);
        }
        super.write(b4);
    }
    
    @Override
    public void derEncode(OutputStream outputStream) throws IOException {
        outputStream.write(this.toByteArray());
    }
    
    static {
        DerOutputStream.lexOrder = new ByteArrayLexOrder();
        DerOutputStream.tagOrder = new ByteArrayTagOrder();
    }
}
